package xml2fol.DataStructures;

import xml2fol.*;

/**
 * A data structure that holds the filename, xpath offset and length for an element. 
 * Filename and XPath are stored as Strings. The offset and length are stored as integers.
 * @author Alan Woodley
 */


public class FOLStruct extends OLStruct implements Comparable<Object> {

    /** the element's filename */
    public String filename; 
    /** the element's xpath */
    public String xpath; 

   /**
    * Creates an FOLStruct with a given filename, xpath, offset and length
    * @param filename - The element's filename
    * @param xpath    - The element's xpath  
    * @param offset   - The element's offset
    * @param length   - The element's length    
    */        
    public FOLStruct(String filename,  String xpath, int offset, int length){
        super(offset, length);
        this.filename = filename;
        this.xpath = xpath;
    }
    
   /**
    * A function that compares the value of two integers
    * @param i - The first integer
    * @param j - The second integer
    * @return - -1 if i is bigger, 0 if they are equal, 1 if i is smaller    
    */            
    public int compareInt(int i, int j){
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
        
        if (i == j ) return EQUAL;
        if (i > j) return AFTER;
        if (i < j) return BEFORE;
        
        return EQUAL; // wont get to
    }


   /**
    * A function to compare this FOLStruct with another FOLStruct
    * @param otherObj - The other OLStruct
    * @return - -1 if otherObj is BEFORE this; 
    *            0 if they are EQUAL; 
    *            1 if otherObj is AFTER this     
    */        
    public int compareTo(FOLStruct other) {
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
           
            
        if ( this == other ) return EQUAL;
            
        // sort the FOLS based upon
        //  filename accending
        //  offset accending (parent -> child)
        //  length decending (parent -> child)
        //  smaller filename (parent -> child)
        int filenameCompare = compareInt(Integer.parseInt(this.filename),Integer.parseInt(other.filename));
        if (filenameCompare == EQUAL){
            int olCompare = super.compareTo(other);
            if (olCompare == EQUAL) {
                if(this.xpath.length() < other.xpath.length()) return BEFORE;
                else return AFTER;
            } else return olCompare;
                
        } else {
            return filenameCompare;
        }
        
        //return EQUAL;
     }
        
        
    /**
    * A function that compares this FOLStruct with another FOLStruct. 
    * @param otherObject - The other FOLStruct.
    * @return - true if (this FOLStruct equal other FOLStruct) otherwise false.
    */
     public boolean equals(Object otherObject) {
        if (this == otherObject) return true;
        if ( !(otherObject instanceof FOLStruct) ) return false;

        FOLStruct other = (FOLStruct) otherObject;
        // return false if any members don't match
        if (!this.filename.contentEquals(other.filename)) return false;            
        if (this.offset != other.offset) return false;            
        if (this.length != other.length) return false;            
        
        return true;
     }


        
    /**
    * A class that overrides equals must also override hashCode.
    * @return - a hashcode
    */
     public int hashCode() {
         int result = HashCodeUtil.SEED;
         result = HashCodeUtil.hash(result, this);
         return result;
      }
    
    /** 
    * Returns a string representation of the class as "filename,xpath,offset,length"
    * @return - A string equal to "filename,xpath,offset,length".
    * @overrides Object.toString
    */
     public String toString(){
        return filename + Global.FOL_DELIM + xpath + Global.FOL_DELIM + offset + Global.FOL_DELIM  + length;
     }

}
