
package xml2fol.DataStructures;

import xml2fol.*;

/**
 * A data structure that holds the offset and length for an element/passage. 
 * The offset and length are stored as integers.
 * @author Alan Woodley
 */


public class OLStruct implements Comparable<Object>{

    /** the passage's offset */
    public int offset; 
    /** the passage's length */    
    public int length; 
    
    
   /**
    * Creates an OLStruct with a given offset and length
    * @param offset - The passage's offset
    * @param length- The passage's length    
    */    
    public OLStruct(int offset, int length){
        this.offset = offset;
        this.length = length;
    }
    
    
   /**
    * A function that compares the value of two integers
    * @param i - The first integer
    * @param j - The second integer
    * @return - -1 if i is bigger, 0 if they are equal, 1 if i is smaller    
    */        
   public int compareInt(int i, int j){
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
        
        if (i == j ) return EQUAL;
        if (i > j) return AFTER;
        if (i < j) return BEFORE;
        
        return EQUAL; // wont get to
    }


   /**
    * A function to compare this OLStruct with another OLStruct
    * @param otherObj - The other OLStruct
    * @return - -1 if otherObj is BEFORE this; 
    *            0 if they are EQUAL; 
    *            1 if otherObj is AFTER this     
    */        
    public int compareTo(Object otherObj) {
        OLStruct other = (OLStruct) otherObj;
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
           
            
        if ( this == other ) return EQUAL;
            
        // sort the FOLS based upon
        //  offset accending (parent -> child)
        //  length decending (parent -> child)
        int offsetCompare = compareInt(this.offset, other.offset);
        if (offsetCompare == EQUAL){
              return compareInt(other.length, this.length);
           } else {
             return offsetCompare;
        }                
        
        //return EQUAL;
     }
        
        
    /**
    * A function that compares this OLStruct with another OLStruct. 
    * @param otherObject - The other OLStruct.
    * @return - true if (this OLStruct equal other OLStruct) otherwise false.
    */
    public boolean equals(Object otherObject) {
        if (this == otherObject) return true;
        if ( !(otherObject instanceof OLStruct) ) return false;

        OLStruct other = (OLStruct) otherObject;
        // return false if any members don't match
        if (this.offset != other.offset) return false;            
        if (this.length != other.length) return false;            
        
        return true;
    }

        
    /**
    * A class that overrides equals must also override hashCode.
    * @return - a hashcode
    */
    public int hashCode() {
         int result = HashCodeUtil.SEED;
         result = HashCodeUtil.hash(result, this);
         return result;
     }

    
    /** 
    * Returns a string representation of the class as "offset,length"
    * @return - A string equal to "offset,length".
    * @overrides Object.toString
    */
    public String toString(){
        return offset + Global.FOL_DELIM + length;
    }
    
    
}
