
package xml2fol.DataStructures;

/**
 * A data structure that holds the offset and length for an element/passage. 
 * The offset and length are stored as longs. 
 * @author Alan Woodley
 */

import xml2fol.*;
import java.util.Collections;

public class OLStructLong implements Comparable<Object>{

    /** the passage's offset */
    public long offset;
    /** the passage's length  */ 
    public long length; 
    
   /**
    * Creates an OLStructLong with a given offset and length
    * @param offset - The passage's offset
    * @param length- The passage's length    
    */    
    public OLStructLong(long offset, long length){
        this.offset = offset;
        this.length = length;
    }
    
    
   /**
    * A function that compares the value of two longs
    * @param i - The first long
    * @param j - The second long
    * @return - -1 if i is bigger, 0 if they are equal, 1 if i is smaller    
    */            
   public int compareLong(long i, long j){
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
        
        if (i == j ) return EQUAL;
        if (i > j) return AFTER;
        if (i < j) return BEFORE;
        
        return EQUAL; // wont get to
    }

   /**
    * A function to compare this OLStructLong with another OLStructLong
    * @param otherObj - The other OLStructLong
    * @return - -1 if otherObj is BEFORE this; 
    *            0 if they are EQUAL; 
    *            1 if otherObj is AFTER this     
    */        
    public int compareTo(Object otherObj) {
        OLStructLong other = (OLStructLong) otherObj;
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
           
            
        if ( this == other ) return EQUAL;
            
        // sort the FOLS based upon
        //  offset accending (parent -> child)
        //  length decending (parent -> child)
        int offsetCompare = compareLong(this.offset, other.offset);
        if (offsetCompare == EQUAL){
              return compareLong(other.length, this.length);
           } else {
             return offsetCompare;
        }                
        
        //return EQUAL;
     }
        
        
    /**
    * A function that compares this OLStructLong with another OLStructLong. 
    * @param otherObject - The other OLStructLong.
    * @return - true if (this OLStructLong equal other OLStructLong) otherwise false.
    */
    public boolean equals(Object otherObject) {
        if (this == otherObject) return true;
        if ( !(otherObject instanceof OLStructLong) ) return false;

        OLStructLong other = (OLStructLong) otherObject;
        // return false if any members don't match
        if (this.offset != other.offset) return false;            
        if (this.length != other.length) return false;            
        
        return true;
    }

        
    /**
    * A class that overrides equals must also override hashCode.
    * @return - a hashcode
    */
    public int hashCode() {
         int result = HashCodeUtil.SEED;
         result = HashCodeUtil.hash(result, this);
         return result;
     }

    
    /** 
    * Returns a string representation of the class as "offset,length"
    * @return - A string equal to "offset,length".
    * @overrides Object.toString
    */
    public String toString(){
        return offset + Global.FOL_DELIM + length;
    }
    
    
}
