package xml2fol.DataStructures;

/**
 * A data structure to store details about each element or range result
 * 3 types of results are stored using this data structure
 * Elements - submission should specify subName, start, elementRank, Filename (end = start, start_index/end_index = 0)
 * Ranges - submission should specify subName, start, end, elementRank, Filename (start_index/end_index = 0)
 * Ranges with Text Offset - submission should specify subName, start, start_index, end, end_index, elementRank, Filename 
 */

import java.io.*;
import java.util.*;

public class XPathRankStruct {
    
    /** The name of the submission that returned this result */
    public String subName;
    /** The start element for the result (if its a range) */
    public String start; 
    /** The offset of the start element (if it is a text element, 0 otherwise)  */  
    public int start_index;
    /** The end element for the result (NB if its a XPath result then end == start) */
    public String end; 
    /** The offset of the start element (if it is a text element, 0 otherwise) */
    public int end_index;  
    /** The rank of the result in terms of elements - used to calculate pool depth in terms of elements     */
    public int elementRank; 
    /** The rank of the result in terms of files - used to calculate pool depth in terms of files */
    public int fileRank; 
    
    
    /**
    * Produces a XPathRankStruct with the following parameters
    * @param subName - The name of the submission that returned this result
    * @param start - The start element for the result (if its a range)
    * @param start_index - The offset of the start element (if it is a text element, 0 otherwise)
    * @param end - The end element for the result (NB if its a XPath result then end == start)
    * @param end_index - The offset of the start element (if it is a text element, 0 otherwise)
    * @param elementRank - The rank of the result in terms of elements - used to calculate pool depth in terms of elements 
    * @param fileRank - The rank of the result in terms of files - used to calculate pool depth in terms of files 
    */
    public XPathRankStruct(String subName, String start, int start_index, String end, int end_index, int elementRank, int fileRank){
        this.subName = subName;
        this.start = start;
        this.start_index =  start_index;
        this.end = end;
        this.end_index =  end_index;
        this.elementRank = elementRank;
        this.fileRank = fileRank;

    }
    
    

}
