/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package xml2fol;

import xml2fol.DataStructures.FOLStruct;
import java.io.*;
import java.util.HashMap;
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import java.util.Hashtable;
import org.xml.sax.InputSource;
//import org.xml.sax.SAXException;

/**
 *
 * @author Alan
 */
public class FOLTable {

    int minCharacterLength; // only include elements of this charcater size
    String fxolFilename;
    String idxFilename;
    int currentOffset; // offset into the fxol collection
    boolean useIndex;
    boolean saveText; 
    String saveType;
    DataOutputStream data_out;

    protected Document doc;
    

    /**
     * Creates an FOLTable with the following parameters 
     *     No Index Filename (filename is empty string)
     *     Values are stored in both Map and List
     *     Min character length is 0     
     * 
     * @param fxolFilename - The name of the FOL table
     * @param tableFlag - A flag to indicate the type of table created
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String tableFlag, boolean saveText, String saveType){
        this(fxolFilename, "", tableFlag, 0, true, true, saveText, saveType);
    }
    
    
    /**
     * Creates an FOLTable with the following parameters 
     *     Values are stored in both Map and List
     *     Min character length is 0
     *  
     * @param fxolFilename - The name of the FOL table
     * @param idxFilename - The name of the FOL table's index
     * @param tableFlag - A flag to indicate the type of table created
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String idxFilename, String tableFlag, boolean saveText, String saveType){
        this(fxolFilename, idxFilename, tableFlag, 0, true, true, saveText, saveType);
    }

    /**
     * Creates an FOLTable with the following parameters 
     *     No Index Filename (filename is empty string)
     *     Values are stored in both Map and List
     * 
     * @param fxolFilename - The name of the FOL table
     * @param tableFlag - A flag to indicate the type of table created
     * @param minCharacterLength - The minimum character length required for elements to be stored in the table
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String tableFlag, int minCharacterLength, boolean saveText, String saveType){
        this(fxolFilename, "", tableFlag, minCharacterLength, true, true, saveText, saveType);
    }

    /**
     * Creates an FOLTable with the following parameters 
     *     Values are stored in both Map and List
     * 
     * @param fxolFilename - The name of the FOL table
     * @param idxFilename - The name of the FOL table's index
     * @param tableFlag - A flag to indicate the type of table created
     * @param minCharacterLength - The minimum character length required for elements to be stored in the table
     * @param saveText - A flag to indicate if text elements should be stored in the table
     *
     */
    public FOLTable(String fxolFilename, String idxFilename, String tableFlag, int minCharacterLength, boolean saveText, String saveType){
        this(fxolFilename, idxFilename, tableFlag, minCharacterLength, true, true, saveText, saveType);
    }

    /**
     * Creates an FOLTable with the following parameters 
     *     Min character length is 0
     *     No Index Filename (filename is empty string)
     * 
     * @param fxolFilename - The name of the FOL table
     * @param tableFlag - A flag to indicate the type of table created
     * @param storeMap - A flag to indicate if values should be store in the Map
     * @param storeList - A flag to indicate if values should be store in the Map
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String tableFlag, boolean storeMap, boolean storeList, boolean saveText, String saveType){
        this(fxolFilename, "", tableFlag, 0, storeMap, storeList, saveText, saveType);
    }
    
    /**
     * Creates an FOLTable with the following parameters 
     *     Min character length is 0
     * 
     * @param fxolFilename - The name of the FOL table
     * @param idxFilename - The name of the FOL table's index
     * @param tableFlag - A flag to indicate the type of table created
     * @param storeMap - A flag to indicate if values should be store in the Map
     * @param storeList - A flag to indicate if values should be store in the Map
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String idxFilename, String tableFlag, boolean storeMap, boolean storeList, boolean saveText, String saveType){
        this(fxolFilename, idxFilename, tableFlag, 0, storeMap, storeList, saveText, saveType);
    }

    /**
     * Creates an FOLTable with the following parameters 
     *    No Index Filename (filename is empty string)
     * 
     * @param fxolFilename - The name of the FOL table
     * @param tableFlag - A flag to indicate the type of table created
     * @param minCharacterLength - The minimum character length required for elements to be stored in the table
     * @param storeMap - A flag to indicate if values should be store in the Map
     * @param storeList - A flag to indicate if values should be store in the Map
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */    
    public FOLTable(String fxolFilename, String tableFlag, int minCharacterLength, boolean storeMap, boolean storeList, boolean saveText, String saveType){
        this(fxolFilename, "", tableFlag, minCharacterLength, storeMap, storeList, saveText, saveType);

    }
    
    /**
     * Creates an FOLTable with the following parameters 
     * 
     * @param fxolFilename - The name of the FOL table
     * @param idxFilename - The name of the FOL table's index
     * @param tableFlag - A flag to indicate the type of table created
     * @param minCharacterLength - The minimum character length required for elements to be stored in the table
     * @param storeMap - A flag to indicate if values should be store in the Map
     * @param storeList - A flag to indicate if values should be store in the Map
     * @param saveText - A flag to indicate if text elements should be stored in the table
     */
    public FOLTable(String fxolFilename, String idxFilename, String tableFlag, int minCharacterLength, boolean storeMap, boolean storeList, boolean saveText, String saveType){
        this.minCharacterLength = minCharacterLength;
//        tableHandler = TableHandlerFactory.getTableHandler(fxolFilename, idxFilename, tableFlag);
        this.fxolFilename = fxolFilename;
        this.idxFilename = idxFilename;
        currentOffset = 0;
        useIndex = !(idxFilename.contentEquals(""));
        this.saveText = saveText;
        this.saveType = saveType;
    }
           
    /**
     * Adds a given xmlfile (and all its xpaths) to the table
     * @param xmlfilename
     * @return - The length of the file in bytes
     */
    public int AddFile(String xmlfilename){
        String filename = Global.GetJustFileName(xmlfilename); 
        try {
           // load the XML document into memory
          // Set up a DOM tree to support node-text retrieval.
          
          InputSource input = new InputSource(new FileInputStream(xmlfilename));
          DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
          DocumentBuilder builder = dfactory.newDocumentBuilder();
          doc = builder.parse(input);

          // produce the xpath positions
          int start=0; // start position of node text in the text-only content
          int index = 1; // start index of path element
          
          // traverse the file collecting data about each node
          int documentLength = traverseAllNodes(filename, "",doc.getFirstChild(),index,start);
          
//          // append the fol data to the  file
//          WriteFileToTable(xmlfilename);
          
          // clear the data structures (for memory managaement)
//          fxolCollection.Clear();
//          idxCollection.Clear();
          
          return documentLength;

        } catch (Exception e) {
            System.out.println(xmlfilename);
            e.printStackTrace();
        }

        return -1;
        
        
    }
        
    /**
     * Function used to traverse all the nodes in a given XML File
     * @param filename - The name of the XML  file
     * @param xpath - The name of the current XPath being traversed
     * @param node - The Node being traversed
     * @param idx - The in-order number of siblings with the same name as this element (including this element i.e. indexed started from 1)
     * @param start - The number of bytes in the files before this element
     * @return - Start + Length of this element in bytes
     */
    private int traverseAllNodes(String filename, String xpath,Node node,int idx,int start){

        // skip comment nodes
        while (node.getNodeType()!=node.ELEMENT_NODE && node.getNodeType()!=node.TEXT_NODE) {
            node = node.getNextSibling(); // process only elements and their text
            if (node==null)
                return 0;
        }
        String s  ="";
        int offset=start;
        if (node.getNodeType()==node.ELEMENT_NODE) {
            s = node.getTextContent();
            xpath+="/"+node.getNodeName()+"["+idx+"]";   
        } else if (node.getNodeType()==node.TEXT_NODE) {
            // if we have a text node add it to the xapth hashtable
            s = node.getTextContent();
            xpath += "/text()" +"["+idx+"]";
        //        System.out.println("Text Found at" + xpath);
            int length = s.length();
            offset += length;
        
            // add node to list is text is meant to be saved
            // only add characters of a given size
            if(saveText && ((length >= minCharacterLength) || (xpath.endsWith("/name[1]")))){
                FOLStruct fxol = new FOLStruct(filename,xpath, start, length);
//                String fxolKey = Global.GetKey(filename, xpath);
//                fxolCollection.Add(fxolKey, fxol); /*** stuffup ***/
                if (this.saveType.equals("binary")) {
                    try {
                        data_out.writeLong(Global.GetHashKey(fxol.filename, fxol.xpath));
                        data_out.writeInt (fxol.offset);
                        data_out.writeInt (fxol.length);                    
                    }
                    catch (IOException e) {
                       System.out.println ("IO exception = " + e );
                    }    
                } else { // type is text
                    try {
                        if (!XML2FOL.textout) {
                            data_out.writeBytes(fxol.toString()+"\n");
                        } else{
                            s=s.replaceAll("\\s"," "); // replace white space chars by space
                            data_out.writeBytes(fxol.toString()+","+s+"\n");                            
                        }
                    }
                    catch (IOException e) {
                       System.out.println ("IO exception = " + e );
                    }    
                }
            }        
        
            return offset;
        } if(node.hasChildNodes()) { 
           Hashtable nodeNames = new Hashtable();
           NodeList NL=node.getChildNodes();            
           for(int i=0;i<NL.getLength();i++) {
                String nodeName = NL.item(i).getNodeName();
                if (nodeNames.containsKey(nodeName)) {
                     // here if the node name is already in the table
                    Integer ix = (Integer) nodeNames.get(nodeName);
                    ix++;
                    nodeNames.put(nodeName, ix);
                    offset = traverseAllNodes(filename,xpath,NL.item(i),ix,offset);
                }  else {
                    // here if node name encountered for the first time
                    nodeNames.put(nodeName, new Integer(1));
                    offset = traverseAllNodes(filename,xpath,NL.item(i),1,offset);
                }
          }  // for nodelist
     }
     // add node to list

     int length = node.getTextContent().length();
     // only add characters of a given size
     if(length >= minCharacterLength || xpath.endsWith("/name[1]")){
        FOLStruct fxol = new FOLStruct(filename,xpath, start, length);
//        String fxolKey = Global.GetKey(filename, xpath);
//        fxolCollection.Add(fxolKey, fxol); /*** stuffup ***/
        if (this.saveType.equals("binary")) {
            try {
                data_out.writeLong(Global.GetHashKey(fxol.filename, fxol.xpath));
                data_out.writeInt (fxol.offset);
                data_out.writeInt (fxol.length);                    
            }
            catch (IOException e) {
               System.out.println ("IO exception = " + e );
            }    
        } else { // type is text
            try {
                if (!XML2FOL.textout) {
                    data_out.writeBytes(fxol.toString()+"\n");
                } else{
                    s=s.replaceAll("\\s"," "); // replace white space chars by space
                    data_out.writeBytes(fxol.toString()+","+s+"\n");                            
                }
            }
            catch (IOException e) {
               System.out.println ("IO exception = " + e );
            }    
        }
     }
     return offset;
   } 
    /**
     * Counts tags in a given xmlfile
     * @param xmlfilename
     * @return - a map collection of tags
     */
    public HashMap collectTags(String xmlfilename){
        HashMap map = new HashMap();    // hash table
        try {
           // load the XML document into memory
          // Set up a DOM tree to support node-text retrieval.

          InputSource input = new InputSource(new FileInputStream(xmlfilename));
          DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
          DocumentBuilder builder = dfactory.newDocumentBuilder();
          doc = builder.parse(input);

          // traverse the file collecting data about each node
          map = traverseNodes(map, doc.getFirstChild());

          return map;

        } catch (Exception e) {
            System.out.println("*** Could not parse "+xmlfilename+" ***");
            e.printStackTrace();
        }

        return map;


    }

    /**
     * Function used to traverse all the nodes in a given XML File
     * @param filename - The name of the XML  file
     * @param xpath - The name of the current XPath being traversed
     * @param node - The Node being traversed
     * @param idx - The in-order number of siblings with the same name as this element (including this element i.e. indexed started from 1)
     * @param start - The number of bytes in the files before this element
     * @return - Start + Length of this element in bytes
     */
    private HashMap traverseNodes(HashMap map, Node node){

        // skip comment nodes
        while (node.getNodeType()!=node.ELEMENT_NODE && node.getNodeType()!=node.TEXT_NODE) {
            node = node.getNextSibling(); // process only elements and their text
            if (node==null)
                return map;
        }
        String s  ="";
        int count;
        if (node.getNodeType()==node.ELEMENT_NODE) {
            s = node.getNodeName(); // get tag
            if (s.length()>60)
                System.out.println("oops! : "+s);
            if (map.containsKey(s)) {
                count = (Integer) map.get(s) + 1;
                map.put(s, count);
            } else {
                map.put(s, 1);
            }


        }
        if(node.hasChildNodes()) {
           NodeList NL=node.getChildNodes();
           for(int i=0;i<NL.getLength();i++) {
                    map = traverseNodes(map,NL.item(i));
           }  // for nodelist
        }
        return map;
   }
               
}
