package xml2fol;

import java.io.*;
import java.util.*;
import java.nio.*;
import java.nio.channels.*;       

/**
 * A class that performs comman file utilities
 * @author Alan Woodley
 */
public class FileUtil {

    
   /**
    * A method that moves files from one location to another.
    * If a directory is specified as inname then all files in the directory will be moved.
    * @param inname - source file/directory
    * @param outtname - destination file/directory
    */            
   public static void MoveRecursive(String inname, String outname){
//       System.out.println("Moving " + inname + " to " + outname);
        inname = inname.toLowerCase().trim();
        File in = new File(inname);
        if (in.isFile()){
            File out = new File(outname);
            boolean renameValue = in.renameTo(out);
            // sometimes rename doesn't work, so if the file has sucessfully been copied
            // but the original file has not been deleted (i.e. there are 2 versions of the files)
            // then remove the original
            if(in.exists() && out.exists()) in.delete();
//            System.out.println("Rename : " + renameValue);
        } else if (in.isDirectory()){
            // if we have a directory then process all of its file and subdirectories
            if (!inname.endsWith(File.separator)) inname += File.separator;
            if (!outname.endsWith(File.separator)) outname += File.separator;
//            System.out.println("Moving directory " + inname);
            String[] children = in.list();
            for(int i=0;i<children.length;i++){
                MoveRecursive(inname + children[i], outname  + children[i]);
            }
            in.delete();
        } else {
            // do nothing           
        }
        
    }     
   
   /**
    * A method that deletes files.
    * If a directory is specified as inname then all files in the directory will be deleted.
    * @param inname - source file/directory
    */            
    public static void DeleteRecursive(String name){
        File file = new File(name);
        if (file.isFile()){
//            System.out.println("Deleting " + name + "  ");           
            file.delete();
        } else if (file.isDirectory()){
            // if we have a directory then process all of its file and subdirectories
            if (!name.endsWith(File.separator)) name += File.separator;
            String[] children = file.list();
            for(int i=0;i<children.length;i++){
                DeleteRecursive(name + children[i]);
            }
            file.delete();
        } else {
            // do nothing           
        }
        
    }      
    
        
   /**
    * A method that copied files from one location to another.
    * If a directory is specified as inname then all files in the directory will be copied.
    * @param inname - source file/directory
    * @param outtname - destination file/directory
    */            
    public static void CopyRecursive(String inname, String outname){
//        System.out.println("Copying " + inname + " to " + outname);
        File infile = new File(inname);
        if(!infile.exists()) {
            System.out.println("Error file " + inname + " not found.");
            return;
        }
        if (infile.isFile()){
            File outfile = new File(outname);
            copy(infile, outfile);
        } else if (infile.isDirectory()){
            // if we have a directory then process all of its file and subdirectories
            if (!inname.endsWith(File.separator)) inname += File.separator;
            if (!outname.endsWith(File.separator)) outname += File.separator;
            String[] children = infile.list();
            for(int i=0;i<children.length;i++){
                CopyRecursive(inname + children[i], outname + children[i]);
            }
        } else {
            // do nothing           
        }
        
    }     
            
    /**
     * Creates the directory structure for a filename if it does not alreay exist
     * 
     * @param filename - the name of the file to create
     */
    public static void CreateDirsNotExists(String filename){
        try {
            File file = new File(filename);
            if(!file.exists()) {
                File parentDir = file.getParentFile();
                if(!parentDir.exists()) parentDir.mkdirs();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Creates a mirror set of sub directories in a source directory in 
     * a destination directory
     * @param String
     */
    public static void CreateMirrorDirsNotExists(String inname, String outname){
        try {
              
            
//            System.out.println("looking at file " + inname);
            File infile = new File(inname);
            // if we have a directory
            if(infile.isDirectory()){
                inname = inname.replace("/", File.separator);
                outname = outname.replace("/", File.separator);
                if(!inname.endsWith(File.separator)) inname = inname + File.separator;
                if(!outname.endsWith(File.separator)) outname = outname + File.separator;     
//                System.out.println("Creating directory " + outname);
                // first create destination
                File outfile = new File(outname);
                if(!outfile.exists()) outfile.mkdirs();

                //  then create the mirror of the source in the destination
                String[] children = infile.list();
                for(int i=0;i<children.length;i++){
                    CreateMirrorDirsNotExists(inname + children[i], outname + children[i]);
                }                        
                         
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
   /**
    * Method that physcially copies the files 
    * @param source - source file
    * @param dest - destination file
    * @throws java.io.IOException
    */
    private static void  copy(File source, File dest) {
        //       System.out.println("Copying " + source + " to " + dest);
        FileChannel in = null, out = null;
        try {          
            in = new FileInputStream(source).getChannel();
            out = new FileOutputStream(dest).getChannel();
 
            long size = in.size();
            MappedByteBuffer buf = in.map(FileChannel.MapMode.READ_ONLY, 0, size);
 
            out.write(buf);

         } catch (Exception e){
             e.printStackTrace();
         } finally {
            try {
                if (in != null)      in.close();
                if (out != null)     out.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

     
            
}
