package xml2fol;

//import xml2fol.TableHandlers.TableHandlerFactory;
import java.io.*;
import java.util.*;
import java.util.Hashtable;

/**
 * This is a class that saves the offsets and lengths (in bytes) for each of the
 * elements in collection.
 *
 * To use this class you need xalan.jar 
 * http://xml.apache.org/xalan-j/
 * @author Alan Woodley
 * @butcher Shlomo Geva
*/
public class XML2FOL {

    // Command Line Arguments
    private static final String TYPE_CLA = "type"; // flag to indicate type of table saved (binary or text) 
    private static final String MINBYTES_CLA = "minbytes"; // flag to indicate the length an element must be to include it in the table 
    private static final String TEXTOUT_CLA = "textout"; // flag to indicate the table should ibnclude node text 
    private static final String SAVE_TEXT_CLA = "savetext";// flag to indicate if text elements should be saved in the table or not

    /** the directory of the XML collection */
    String input;
    /** the table to save the FOLs */    
    String table;
    /** the index table to save the FOLs */    
    String index="";
    /** a flag to indicate if values should be saved in a Map */       
    boolean saveMap;
    /** a flag to indicate if values should be saved in a List */       
    boolean saveList;
    /** the minimum number of bytes that an element requires if it should be saved in the list */       
    int minElementLength;
    /** the type of table to be saved */           
    public String type;
    /** a flag to indicate if the table should include node text */           
    static boolean textout;
    /** a flag to indicate if text elements should be saved in the table or not */           
    boolean saveText;    
    /** the collection */               
    FOLTable collection;
    FileOutputStream file_output;    
    static int fileCount=0;
    static long startTime;
    HashMap TagMap = new HashMap();
    /**
      * Creates a XMl2FOL object without an input, table or index value,
      * only saves values to the list (not the map), sets the minimum length
      * to 0, saves text elements in the table sets the table type to binary,
      * and sets textout to false.  
      * These values can be modifed by command line arguments.
      */
    public XML2FOL(){
        input = "";
        table = "";
        index = "";
        saveMap = false; // don't save the entries to a hashtable (for now)
        saveList = true; // save the entries to a list
        minElementLength = 50;
        type = "text";
        textout = false;        
        saveText = false;
    }

    private class Frequency {
        int FileFreq;
        int CollectionFreq;
        
        public Frequency(int f, int c) {
            FileFreq = f;
            CollectionFreq = c;
        }
    }
    private void addTags(HashMap map) {
//                    if (map.containsKey(s)) {
//                count = (Integer) map.get(s) + 1;
//                map.put(s, new Integer(count));
//            } else {
//                map.put(s, new Integer(1));
//            }
    // Iterate over the keys in the map
        if (map==null) {
            System.out.println("Empty map?");
            return;
        }
        Frequency f;
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            // Get key
            Object key = it.next();
            if (TagMap.containsKey(key)) {
                f = (Frequency) TagMap.get(key);
                f.FileFreq++;
                f.CollectionFreq += (Integer) map.get(key);
            } else {
                TagMap.put(key, new Frequency(1,(Integer) map.get(key)));
            }
        }
    }
    
    private void writeTags() {
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter("D:/tmp/INEXWikiTags.txt"));
            out.write("aString");
            Frequency f;
            Iterator it = TagMap.keySet().iterator();
            while (it.hasNext()) {
                // Get key
                Object key = it.next();
                f = (Frequency) TagMap.get(key);
                out.write((String) key + "," + f.FileFreq + "," + f.CollectionFreq);
                out.newLine();
            }
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
      * Recursively gets the offset and lengths for the elements in a
      * document or directory of documents.
      * @param inname - the name of the directory or file
      */    
    private void ConvertXMLRecursive(String inname){
//        inname = inname.toUpperCase();
        File in = new File(inname);
        if (in.isFile()){
            // if we have an XML file then process it
            if (inname.toUpperCase().endsWith(Global.XML_SUFFIX)) {
                // add the file to the collection  

                ++fileCount;
//                addTags(collection.collectTags(inname));
                
                collection.AddFile(inname);                   
                if ((fileCount%1000)==0) {
                    System.out.print(fileCount+", ");
                    long after = System.currentTimeMillis() ;
                    long elapsed = after - startTime;
                    System.out.println(fileCount*100.0/2666000.0 + "% converted.  Elapsed time : "+(elapsed/60000.0)+" minutes"+", "+TagMap.size()+" tags");
                }
            }
        } else if (in.isDirectory()){
            // if we have a directory then process all of its file and subdirectories
            if (!inname.endsWith(File.separator)) inname += File.separator;
            long after = System.currentTimeMillis() ;
            long elapsed = after - startTime;
            System.out.println("Converting directory " + inname+" , "+fileCount+"("+fileCount*100.0/2666000.0 + "%) converted.  Elapsed time : "+(elapsed/60000.0)+" minutes"+", "+TagMap.size()+" tags");
            String[] children = in.list();
            for(int i=0;i<children.length;i++){
                ConvertXMLRecursive(inname + children[i]);
            }
        } else {
            // do nothing           
        }
        
    }       

    /**
      * Public interface. Gets the offset and lengths for the elements in a
      * docuument or directory of documents.
      * @param inname - the name of the directory or file
      */    
    public void ConvertXML(String inname){
        inname = inname.replace('/', File.separatorChar);
        try {
            // Create an output stream to the file.
            // Wrap the FileOutputStream with a DataOutputStream
            file_output = new FileOutputStream (table);
            collection.data_out = new DataOutputStream (file_output);

            ConvertXMLRecursive(inname);
            // Close file when finished with it..
            file_output.close ();
            System.out.println("Converted "+fileCount+" files");
        }
        catch (IOException e) {
           System.out.println ("IO exception = " + e );
        }
        writeTags();
 
    }
    
    
    
    /**
     * Proceses the command line arguments
     * @param args - The array of command line arguments
     */
    public void ProcessArguments(String args[]){
          // Usage: java -jar xml2fol.jar [-type={text|binary}]  [-savetext={true|false}] [-minchars=length] [-textout={true|false}] input table"

        // first get the values for the directory of xml documents (input) and for the output table
        Hashtable otherCmds = new Hashtable();
        if(args.length < 2){
            System.out.println("Error - Insufficient number of arguments.");
            System.out.println();
            XML2FOL.PrintStatusMessage();
            System.exit(-1);
        } else if (args.length == 2) {
            input = args[0];
            table = args[1];
        } else if (args.length == 3) {
            input = args[1];
            table = args[2];
            otherCmds = Global.ProcessCommandsLinrArguments(Arrays.copyOfRange(args, 0, 1));
        } else if (args.length == 4) {
            input = args[2];
            table = args[3];
            otherCmds = Global.ProcessCommandsLinrArguments(Arrays.copyOfRange(args, 0, 2));            
        } else if (args.length == 5) {
            input = args[3];
            table = args[4];
            otherCmds = Global.ProcessCommandsLinrArguments(Arrays.copyOfRange(args, 0, 3));
        } else if (args.length == 6) {
            input = args[4];
            table = args[5];
            otherCmds = Global.ProcessCommandsLinrArguments(Arrays.copyOfRange(args, 0, 4));
        } else {
            System.out.println("Error - Incorrect number of arguments.");
            System.out.println();
            XML2FOL.PrintStatusMessage();
            System.exit(-1);        
        }

        // then process the other arguments
        
        // check to see if the user has specified a 'type' argument        
        if(otherCmds.containsKey(TYPE_CLA)){
            type = (String) otherCmds.get(TYPE_CLA);

            // check to see that type is valid
            if (!((type.contentEquals("text")) || (type.contentEquals("binary")))){
                System.out.println("Error - Incorrect table type.");
                System.out.println();
//                XML2FOL.PrintStatusMessage();
                System.exit(-1);        
            }
            otherCmds.remove(TYPE_CLA);
            
        } 

        // check to see if the user has specified a 'minchars' argument        
        if(otherCmds.containsKey(MINBYTES_CLA)){
            minElementLength = Integer.parseInt((String) otherCmds.get(MINBYTES_CLA));  
            otherCmds.remove(MINBYTES_CLA);
        }

        // check to see if the user has specified a 'textout' argument        
        if(otherCmds.containsKey(TEXTOUT_CLA)){
            textout = Boolean.parseBoolean((String) otherCmds.get(TEXTOUT_CLA)); 
             // unable to output text to binary tables
            if(textout && (!type.contentEquals("text"))){
                System.out.println("Error - Unable to output text with Binary Table. Choose to output as *Text*");
                System.out.println();
//                XML2FOL.PrintStatusMessage();
                System.exit(-1);                
            }
            otherCmds.remove(TEXTOUT_CLA);
        }        
        
        // check to see if the user has specified that text elements should be included 
        if(otherCmds.containsKey(SAVE_TEXT_CLA)){
            saveText = Boolean.parseBoolean((String) otherCmds.get(SAVE_TEXT_CLA)); 
            otherCmds.remove(SAVE_TEXT_CLA);
        }    
        
        // check to see if there are any more arguments left - if so they have entered
        // an illegal argument
        if(otherCmds.size() != 0){
            
            for(Enumeration<String> keys = (otherCmds.keys()); keys.hasMoreElements();){
                String k  = (String) keys.nextElement();
                System.out.println("Error - Unknown paramter \"" + k + "\"");
            }      
          
            System.out.println("conversion not successful");
            System.exit(-1);
        }
        
        // check that the input file or directory exists
        File inFile = new File(this.input);
        if(!inFile.exists()){
             System.out.println("Error - Input file " + inFile + " does not exist");
             System.out.println("conversion not successful");
             System.exit(-1);
        }
        
        
//        System.out.println("Last index of " +  + " in table " + this.table + ":" + this.table.lastIndexOf("/"));
        if(this.table.contains(File.separator)){
            File tableFilePath =  new File(this.table.substring(0,this.table.lastIndexOf(File.separator)));
            if(!tableFilePath.exists()) tableFilePath.mkdirs();        
        }
        
        if(this.index.contains(File.separator)){
            File indexFilePath =  new File(this.index.substring(0,this.index.lastIndexOf(File.separator)));
            if(!indexFilePath.exists()) indexFilePath.mkdirs();        
        }
        
        System.out.println("input " + input + ", Table " + table + ", type " + type + ", min " + minElementLength +  ", saveText " + saveText+  ", textOut " + textout);


    }
    
    public void CreateCollection(){
        this.collection = new FOLTable(table, index, type, minElementLength, saveMap, saveList, saveText, type);
    }

    /**
     * Prints a status message on how to use this program
     */
    public static void PrintStatusMessage(){
            System.out.println("XML2FOL Version 1.2 By Alan Woodley, Shlomo Geva");
            System.out.println("This program accepts an XML file (or directory of XML files) and outputs the");
            System.out.println("offsets (number of characters from start of the document to the XPaths)  and ");
            System.out.println("lengths (in number of bytes) of the elements into a table.");
            System.out.println("The FOL table can be saved as a text or binary file. In addition, a subset of the table ");
            System.out.println("can be produced which only includes the XPaths of elemtents over a certain number of characters. ");
            System.out.println("");
            System.out.println("Usage: java -jar xml2fol.jar [-type={text|binary}] [-minbytes=length] [-textout={true|false}]  [-savetext=(true|false)] inputFileName ouptutFileName"); 
            System.out.println("Where: ");
            System.out.println("    inputFileName = The XML File or Directory ");
            System.out.println("    outputFileName = The file containing the table of offsets and lengths"); 
            System.out.println("    Type = optional argument that states if a text or binary file should be created. Valid options are either text or binary (default is text)  ");
            System.out.println("    Minbytes = optional argument that specifies the minimum size in bytes of an element included in the table (default is 50).  Beware, choosing zero will produce a huge table");
            System.out.println("    textout = optional argument that specifies if nodes text should be ouptut.  Default value is false.  Beware - true could generate a large output file - larger than the collection itlself");
            System.out.println("    Savetext = optional argument to indicate if DOM text elements should be saved in the table or not. Default value is false.");
            System.out.println("");
            System.out.println("Conditions: "); 
            System.out.println("    -  Input must exist.");
            System.out.println("    -  Process will only be performed on files with an xml extension.");
            System.out.println("    -  Output will only be appended to (never overwritren).");
    }
    

    
    public static void main(String[] args){

        XML2FOL me = new XML2FOL();
        me.ProcessArguments(args);

        // create collection
        me.CreateCollection();
        startTime = System.currentTimeMillis() ;
        
        // process the input files
        me.ConvertXML(me.input);      

        System.out.print("Prcessed "+fileCount+", ");
        long after = System.currentTimeMillis() ;
        long elapsed = after - startTime;
        System.out.println("elapsed time : "+(elapsed/60000.0)+" minutes");
           
    }
    
}
